#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from pushbullet.pushbullet import PushBullet


def plugin(srv, item):
    ''' expects (apikey, device_id) in adddrs '''

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    recipient_type = "device_iden"
    try:
        apikey, device_id = item.addrs
    except:
        try:
            apikey, device_id, recipient_type = item.addrs
        except:
            srv.logging.warn("pushbullet target is incorrectly configured")
            return False

    text = item.message
    title = item.get('title', srv.SCRIPTNAME)

    try:
        srv.logging.debug("Sending pushbullet notification to %s..." % (item.target))
        pb = PushBullet(apikey)
        pb.pushNote(device_id, title, text, recipient_type)
        srv.logging.debug("Successfully sent pushbullet notification")
    except Exception as e:
        srv.logging.warning("Cannot notify pushbullet: %s" % e)
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2018 The mqttwarn developers
from builtins import object
import os
from lovely.testlayers.util import asbool
#from lovely.testlayers.mongodb import MongoLayer


class ServerLayerMixin(object):
    """
    A test class mixin for controlling server layers from ``lovely.testlayers``.

    Even when wired, just runs them conditionally as they can be
    easily disabled before runtime using ``export SERVERLAYERS=false``.

    """

    @classmethod
    def setup_serverlayers(cls):

        if asbool(os.environ.get('SERVERLAYERS', True)):

            # Spin up a MongoDB instance
            cls.mongolayer = MongoLayer('mongodb.single', storage_port=27017)
            cls.mongolayer.setUp()

    @classmethod
    def teardown_serverlayers(cls):

        if asbool(os.environ.get('SERVERLAYERS', True)):

            # Shut down the MongoDB instance
            cls.mongolayer.tearDown()

    @classmethod
    def setup_class(cls):
        """Setup any state specific to the execution of the whole class"""

        # Setup base infrastructure (MongoDB, etc.)
        cls.setup_serverlayers()

    @classmethod
    def teardown_class(cls):
        """Teardown any state specific to the execution of the whole class"""

        # Tear down base infrastructure (MongoDB, etc.)
        cls.teardown_serverlayers()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Fabian Affolter <fabian()affolter-engineering.ch>'
__copyright__ = 'Copyright 2014 Fabian Affolter'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import syslog


def plugin(srv, item):
    """Transfer a message to a syslog server."""

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s",
                        __file__, item.service, item.target)

    facilities = {
        'kernel' : syslog.LOG_KERN,
        'user'   : syslog.LOG_USER,
        'mail'   : syslog.LOG_MAIL,
        'daemon' : syslog.LOG_DAEMON,
        'auth'   : syslog.LOG_KERN,
        'LPR'    : syslog.LOG_LPR,
        'news'   : syslog.LOG_NEWS,
        'uucp'   : syslog.LOG_UUCP,
        'cron'   : syslog.LOG_CRON,
        'syslog' : syslog.LOG_SYSLOG,
        'local0' : syslog.LOG_LOCAL0,
        'local1' : syslog.LOG_LOCAL1,
        'local2' : syslog.LOG_LOCAL2,
        'local3' : syslog.LOG_LOCAL3,
        'local4' : syslog.LOG_LOCAL4,
        'local5' : syslog.LOG_LOCAL5,
        'local6' : syslog.LOG_LOCAL6,
        'local7' : syslog.LOG_LOCAL7
    }

    options = {
        'pid'    : syslog.LOG_PID,
        'cons'   : syslog.LOG_CONS,
        'ndelay' : syslog.LOG_NDELAY,
        'nowait' : syslog.LOG_NOWAIT,
        'perror' : syslog.LOG_PERROR
    }

    priorities = {
        5        : syslog.LOG_EMERG,
        4        : syslog.LOG_ALERT,
        3        : syslog.LOG_CRIT,
        2        : syslog.LOG_ERR,
        1        : syslog.LOG_WARNING,
        0        : syslog.LOG_NOTICE,
        -1       : syslog.LOG_INFO,
        -2       : syslog.LOG_DEBUG
    }

    title = item.get('title', srv.SCRIPTNAME)
    facility  = facilities[item.addrs[0]]
    option = options[item.addrs[1]]

    priority = priorities[item.get('priority', -1)]
    message = item.message

    try:
        srv.logging.debug("Message is going to syslog facility %s..." \
            % ((item.target).upper()))
        syslog.openlog(title, option, facility)
        syslog.syslog(priority, message)
        srv.logging.debug("Successfully sent")
        syslog.closelog()
    except Exception as e:
        srv.logging.error("Error sending to syslog: %s" % e)
        syslog.closelog()
        return False

    return True



from builtins import str
import xively
import datetime
import requests


def get_datastream(feed, datastream_name):
	try:
		for d in feed.datastreams:
			if d.id == datastream_name:
				return d

		#Strange ... lets explicitly request the datastream
		datastream = feed.datastreams.get(datastream_name)
		return datastream
	except:
		#Ho! we didn't find the datastream, better create him myself
		datastream = feed.datastreams.create(datastream_name)
		return datastream


def plugin(srv, item):
	srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

	config = item.config

	#it's essential to have an apikey
	if type(config) != dict or 'apikey' not in config or not config['apikey']:
		return False

	api = xively.XivelyAPIClient(config['apikey'])


	feed = api.feeds.get(int(item.target))
	now = datetime.datetime.utcnow()

	#lets filter data
	ds = []
	for k,v in list(item.data.items()):
		if k in item.addrs:
			ds.append(xively.Datastream(id=str(k), current_value=str(v), at=now))
	feed.datastreams = ds

	#all set, lets update back to xively
	try:
		feed.update()
	except requests.HTTPError as e:
		srv.logging.error("Xively Error: " + e.strerror)
	return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Jan-Piet Mens <jpmens()gmail.com>'
__copyright__ = 'Copyright 2014 Jan-Piet Mens'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    level = item.addrs[0]

    text = item.message

    levels = {
        'debug': srv.logging.debug,
        'info': srv.logging.info,
        'warn': srv.logging.warning,
        'crit': srv.logging.critical,
        'error': srv.logging.error,
    }

    try:
        levels[level]("%s", text)
    except Exception as e:
        srv.logging.warn("Cannot invoke service log with level `%s': %s" % (level, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from six import string_types

__author__    = 'Morten Høybye Frederiksen <morten()mfd-consult.dk>'
__copyright__ = 'Copyright 2016 Morten Høybye Frederiksen'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from builtins import str

import uuid
from iothub_client import *
from iothub_client_args import *


iothub_clients = {}


def iothub_connect(srv, item, deviceid, devicekey):
    # item.config is brought in from the configuration file
    try:
        hostname = item.config['hostname']
    except Exception as e:
        srv.logging.error("Incorrect target configuration for target=%s: %s", item.target, e)
        return False
    protocol = item.config.get('protocol', 'AMQP')
    timeout = item.config.get('timeout')
    minimum_polling_time = item.config.get('minimum_polling_time')
    message_timeout = item.config.get('message_timeout')

    # Prepare connection to Azure IoT Hub
    connection_string = "HostName=%s;DeviceId=%s;SharedAccessKey=%s" % (hostname, deviceid, devicekey)
    connection_string, protocol = get_iothub_opt(["-p", protocol], connection_string)
    client = IoTHubClient(connection_string, protocol)
    if client.protocol == IoTHubTransportProvider.HTTP:
        if timeout is not None:
            client.set_option("timeout", timeout)
        if minimum_polling_time is not None:
            client.set_option("MinimumPollingTime", minimum_polling_time)
    if message_timeout is not None:
        client.set_option("messageTimeout", message_timeout)
    srv.logging.info("Client: protocol=%s, hostname=%s, device=%s" % (protocol, hostname, deviceid))
    return client


def iothub_send_confirmation_callback(msg, res, srv):
    if res != IoTHubClientConfirmationResult.OK:
        srv.logging.error("Message confirmation: id=%s: %s", msg.message_id, res)
    else:
        srv.logging.debug("Message confirmation: id=%s: %s", msg.message_id, res)


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    # addrs is a list[] containing device id and key.
    deviceid, devicekey = item.addrs

    # Create connection
    try:
        if not deviceid in iothub_clients:
            iothub_clients[deviceid] = iothub_connect(srv, item, deviceid, devicekey)
        client = iothub_clients[deviceid]
    except Exception as e:
        srv.logging.error("Unable to connect for target=%s, deviceid=%s: %s" % (item.target, deviceid, e))
        return False

    # Prepare message
    try:
        if isinstance(item.message, string_types):
            msg = IoTHubMessage(bytearray(item.message, 'utf8'))
        else:
            msg = IoTHubMessage(item.message)
        msg.message_id = str("%s:%s" % (item.target, uuid.uuid4().hex))
    except Exception as e:
        srv.logging.error("Unable to prepare message for target=%s: %s" % (item.target, str(e)))
        return False

    # Send
    try:
        client.send_event_async(msg, iothub_send_confirmation_callback, srv)
        srv.logging.debug("Message queued: id=%s", msg.message_id)
    except Exception as e:
        srv.logging.error("Unable to send to IoT Hub for target=%s: %s" % (item.target, str(e)))
        return False

    return True



# -*- coding: utf-8 -*-
# (c) 2018-2019 The mqttwarn developers
from __future__ import division
from builtins import str
from past.utils import old_div
import time
import pytest
from mqttwarn.util import Struct, Formatter, asbool, parse_cron_options, timeout, sanitize_function_name, load_module, \
    load_function, get_resource_content, exception_traceback
from tests import funcfile


def test_struct():
    data = {'hello': 'world'}
    struct = Struct(**data)
    assert struct.hello == 'world'
    assert struct.get('hello') == 'world'
    assert struct.get('unknown', default=42) == 42
    assert repr(struct) == "<hello: 'world'>"
    assert struct.enum() == data


def test_formatter_basic():
    result = Formatter().format("{foo}", **{u'foo': u'Räuber Hotzenplotz'}).encode('utf-8')
    assert result == b'R\xc3\xa4uber Hotzenplotz'


def test_formatter_json():
    result = Formatter().format("{foo!j}", **{u'foo': {u'bar': u'Räuber Hotzenplotz'}}).encode('utf-8')
    assert result == b'{"bar": "R\\u00e4uber Hotzenplotz"}'


def test_asbool():
    assert asbool(True) is True
    assert asbool(False) is False
    assert asbool(None) is False
    assert asbool('true') is True
    assert asbool('false') is False
    assert asbool('yes') is True
    assert asbool('no') is False
    assert asbool('on') is True
    assert asbool('off') is False

    with pytest.raises(ValueError) as excinfo:
        asbool('Hotzenplotz')
    assert str(excinfo.value) == "String is not true/false: 'hotzenplotz'"


def test_parse_cron_options():
    assert parse_cron_options('60; now=true') == {'now': 'true', 'interval': 60.0}


#@pytest.mark.slow
def test_timeout():

    duration = 0.0005
    below_duration = duration - old_div(duration, 2)
    above_duration = duration + old_div(duration, 2)

    def func():
        time.sleep(duration)
        return 42

    def errfunc():
        raise ValueError('Something went wrong')

    assert timeout(func, timeout_secs=below_duration) is False
    assert timeout(func, timeout_secs=above_duration) == 42

    # FIXME: Better catch and report the exception?
    # FIXME: Derive "timeout_secs" from "duration"
    with pytest.raises(ValueError) as excinfo:
        timeout(errfunc, timeout_secs=0.10, default='foobar')
        excinfo.message == 'Something went wrong'


def test_sanitize_function_name():
    assert sanitize_function_name('foobar()') == 'foobar'
    assert sanitize_function_name('hastme') is None
    assert sanitize_function_name(42) is None
    assert sanitize_function_name(None) is None


def test_load_module():
    module = load_module('mqttwarn/services/file.py')
    assert 'plugin' in dir(module)
    assert module.plugin.__code__.co_argcount == 2
    assert 'srv' in module.plugin.__code__.co_varnames
    assert 'item' in module.plugin.__code__.co_varnames


def test_load_function():

    # Load valid function
    func = load_function(name='foobar', filepath=funcfile)
    assert func is not None

    # Load invalid function, function name does not exist in "funcfile"
    with pytest.raises(RuntimeError) as excinfo:
        load_function(name='unknown', filepath=funcfile)
    assert str(excinfo.value) == "Loading function 'unknown' from '{}' failed".format(funcfile)

    # Load invalid function, "funcfile" does not exist at all
    with pytest.raises(RuntimeError) as excinfo:
        load_function(name='unknown', filepath='unknown.txt')
    assert str(excinfo.value) == "Loading Python code from 'unknown.txt' failed"


def test_get_resource_content():
    payload = get_resource_content('mqttwarn.examples', 'basic/mqttwarn.ini')
    assert '[defaults]' in payload


def test_exception_traceback():

    # Get exception from ``sys.exc_info()``
    try:
        raise ValueError('Something reasonable')

    except Exception as ex:
        tb = exception_traceback()
        assert 'ValueError: Something reasonable' in tb



# -*- coding: utf-8 -*-
# (c) 2018-2019 The mqttwarn developers


def foobar():
    return True


def xform_func(data):
    data['datamap-key'] = 'datamap-value'
    return data


def datamap_dummy(topic):
    return {'datamap-key': 'datamap-value'}


def alldata_dummy(topic):
    return {'alldata-key': 'alldata-value'}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Leonardo Rizzi <l.rizzi()wide-net.org>'
__copyright__ = 'Copyright 2017 Leonardo Rizzi'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from future import standard_library
standard_library.install_aliases()
from builtins import str

import urllib.request, urllib.parse, urllib.error
try:
    import simplejson as json
except ImportError:
    import json


def plugin(srv, item):
    """ addrs: (token, roomid, color, notify) """

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    token   = item.addrs[0]
    roomid  = item.addrs[1]
    color   = item.addrs[2]
    notify  = item.addrs[3]
    timeout = item.config.get('timeout', 60)
    server  = item.config.get('server', 'api.hipchat.com')

    url = 'https://' + str(server) + '/v2/room/' + str(roomid) + '/notification'
    message  = item.message

    try:
        headers = {
                'Content-Type': 'application/json',
                'Authorization': 'Bearer %s' % token}

        datastr = json.dumps({
                'message': message,
                'color': color,
                'message_format': 'html',
                'notify': notify})

        request = urllib.request.Request(url, headers=headers, data=datastr)
        resp = urllib.request.urlopen(request, timeout=timeout)
        data = resp.read()

    except Exception as e:
        srv.logging.warn("Cannot POST %s: %s" % (url, e))
        return False

    return True
